/*
 *
 *		UC16Out.cpp
 *		
 *		part of the Evolution UC-16 driver for BeOS R5
 *
 *		stefan@keindesign.de	http://www.keindesign.de/stefan/
 *
 *		              Copyright (c) 2002 Stefan Werner
 *					
 *		Permission is hereby granted, free of charge, to any person 
 *		obtaining a copy of this software and associated documentation files 
 *		(the "Software"), to deal in the Software without restriction, 
 *		including without limitation the rights to use, copy, modify, merge, 
 *		publish, distribute, sublicense, and/or sell copies of the Software, 
 *		and to permit persons to whom the Software is furnished to do so, 
 *		subject to the following conditions:
 *		The above copyright notice and this permission notice shall be 
 *		included in all copies or substantial portions of the Software.
 *
 *		THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 *		EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 *		MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
 *		NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS 
 *		BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN 
 *		ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
 *		CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 *		SOFTWARE
 *
 */

#include "UC16Out.hpp"
#include "USBMIDI.h"
#include <stdio.h>

UC16Out::UC16Out(const USBEndpoint *point, const char* name)
:BMidiLocalConsumer(name){
	uc16 = point->Device();
	ept = point;
	status_t err = uc16->ControlTransfer (USB_REQTYPE_STANDARD | USB_REQTYPE_ENDPOINT_IN, USB_REQUEST_CLEAR_FEATURE, USB_FEATURE_ENDPOINT_HALT, 0, 0, NULL);
	BMidiLocalConsumer::Register();
}

void UC16Out::NoteOn(uchar channel, uchar note, uchar velocity, bigtime_t time){
		data[0] = 0x19;
		data[1] = B_NOTE_ON | channel;
		data[2] = note;
		data[3] = velocity;
		if (ept)
			ept->BulkTransfer(&data, 4);
}

void UC16Out::NoteOff(uchar channel, uchar note, uchar velocity, bigtime_t time){
		data[0] = 0x18;
		data[1] = B_NOTE_OFF | channel;
		data[2] = note;
		data[3] = velocity;
		if (ept)
			ept->BulkTransfer(&data, 4);
}

void UC16Out::KeyPressure(uchar channel, uchar note, uchar pressure, bigtime_t time){
		data[0] = 0x1a;
		data[1] = B_KEY_PRESSURE | channel;
		data[2] = note;
		data[3] = pressure;
		if (ept)
			ept->BulkTransfer(&data, 4);
}

void UC16Out::ControlChange(uchar channel, uchar controlNumber, uchar controlValue, bigtime_t time){
		data[0] = 0x1b;
		data[1] = B_CONTROL_CHANGE | channel;
		data[2] = controlNumber;
		data[3] = controlValue;
		if (ept)
			ept->BulkTransfer(&data, 4);
}


void UC16Out::ProgramChange(uchar channel, uchar programNumber, bigtime_t time){
		data[0] = 0x1c;
		data[1] = B_PROGRAM_CHANGE | channel;
		data[2] = programNumber;
		data[3] = 0x00;
		if (ept)
			ept->BulkTransfer(&data, 4);
}

void UC16Out::ChannelPressure(uchar channel, uchar pressure, bigtime_t time){
		data[0] = 0x1d;
		data[1] = B_CHANNEL_PRESSURE | channel;
		data[2] = pressure;
		data[3] = 0x00;
		if (ept)
			ept->BulkTransfer(&data, 4);
}

void UC16Out::PitchBend(uchar channel, uchar lsb, uchar msb, bigtime_t time){
		data[0] = 0x1e;
		data[1] = B_PITCH_BEND | channel;
		data[2] = lsb;
		data[3] = msb;
		if (ept)
			ept->BulkTransfer(&data, 4);
}

void UC16Out::SystemExclusive(void* i_data, size_t dataLength, bigtime_t time){
	int counter = 0;
	uchar *sysEx = (uchar*) data;
	while ((dataLength - counter) > 0) {
		switch (dataLength) {
			case 1:
				data[0] = 0x15;
				data[1] = sysEx[counter];
				data[2] = 0x00;
				data[3] = 0x00;
				break;
			case 2:
				data[0] = 0x16;
				data[1] = sysEx[counter];
				data[2] = sysEx[counter + 1];
				data[3] = 0x00;
				break;
			case 3:
				data[0] = 0x17;
				data[1] = sysEx[counter];
				data[2] = sysEx[counter + 1];
				data[3] = sysEx[counter + 2];
				break;
			case 4:
				data[0] = 0x14;
				data[1] = sysEx[counter];
				data[2] = sysEx[counter + 1];
				data[3] = sysEx[counter + 2];
				break;
		}
		if (ept)
			ept->BulkTransfer(&data, 4);
		counter = counter + 4;
	}
}

void UC16Out::SystemCommon(uchar statusByte, uchar data1, uchar data2, bigtime_t time){
		data[0] = 0x12;
		data[1] = 0x20;
		data[2] = data1;
		data[3] = data2;
		if (ept)
			ept->BulkTransfer(&data, 4);
}

void UC16Out::SystemRealTime(uchar statusByte, bigtime_t time){
	// TODO
	BMidiLocalConsumer::SystemRealTime(statusByte, time);
}

void UC16Out::TempoChange(int32 bpm, bigtime_t time){
	// TODO
	BMidiLocalConsumer::TempoChange(bpm, time);
}

void UC16Out::AllNotesOff(bool justChannel, bigtime_t time){
	// TODO
	BMidiLocalConsumer::AllNotesOff(justChannel, time);
}
